@@ -13,18 +13,23 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef QGSGRADUATEDSYMBOLRENDERERV2WIDGET_H
-#define QGSGRADUATEDSYMBOLRENDERERV2WIDGET_H
+#ifndef QGSGRADUATEDSYMBOLRENDERERWIDGET_H
+#define QGSGRADUATEDSYMBOLRENDERERWIDGET_H
 
-#include "qgsgraduatedsymbolrenderer.h"
-#include "qgis.h"
-#include "qgsrendererwidget.h"
-#include "qgsproxystyle.h"
 #include <QStandardItem>
 
-#include "ui_qgsgraduatedsymbolrendererv2widget.h"
+
+#include "qgsgraduatedsymbolrenderer.h"
+#include "qgis_sip.h"
+#include "qgsrendererwidget.h"
+#include "qgsproxystyle.h"
+#include "qgsprocessingwidgetwrapper.h"
+
+#include "ui_qgsgraduatedsymbolrendererwidget.h"
+
 #include "qgis_gui.h"
 
+
 #ifndef SIP_RUN
 /// @cond PRIVATE
 
@@ -54,7 +59,7 @@ class GUI_EXPORT QgsGraduatedSymbolRendererModel : public QAbstractItemModel
     void deleteRows( QList<int> rows );
     void removeAllRows();
     void sort( int column, Qt::SortOrder order = Qt::AscendingOrder ) override;
-    void updateSymbology( bool resetModel = false );
+    void updateSymbology();
     void updateLabels();
 
   signals:
@@ -94,9 +99,9 @@ class GUI_EXPORT QgsGraduatedSymbolRendererWidget : public QgsRendererWidget, pr
     ~QgsGraduatedSymbolRendererWidget() override;
 
     QgsFeatureRenderer *renderer() override;
+    void setContext( const QgsSymbolWidgetContext &context ) override;
 
   public slots:
-    void changeGraduatedSymbol();
     void graduatedColumnChanged( const QString &field );
     void classifyGraduated();
     void reapplyColorRamp();
@@ -120,15 +125,28 @@ class GUI_EXPORT QgsGraduatedSymbolRendererWidget : public QgsRendererWidget, pr
 
     void rowsMoved();
     void modelDataChanged();
-    void refreshRanges( bool reset = false );
+
+    /**
+     * Refreshes the ranges for the renderer.
+     *
+     * The \a reset argument is deprecated and has no effect.
+     */
+    void refreshRanges( bool reset );
 
   private slots:
     void mSizeUnitWidget_changed();
     void methodComboBox_currentIndexChanged( int );
+    void updateMethodParameters();
     void cleanUpSymbolSelector( QgsPanelWidget *container );
     void updateSymbolsFromWidget();
-    void toggleMethodWidgets( int idx );
     void dataDefinedSizeLegend();
+    void changeGraduatedSymbol();
+    void selectionChanged( const QItemSelection &selected, const QItemSelection &deselected );
+    void symmetryPointEditingFinished();
+
+  protected slots:
+
+    void pasteSymbolToSelection() override;
 
   protected:
     void updateUiFromRenderer( bool updateCount = true );
@@ -136,8 +154,6 @@ class GUI_EXPORT QgsGraduatedSymbolRendererWidget : public QgsRendererWidget, pr
     void disconnectUpdateHandlers();
     bool rowsOrdered();
 
-    void updateGraduatedSymbolIcon();
-
     //! Returns a list of indexes for the classes under selection
     QList<int> selectedClasses();
     QgsRangeList selectedRanges();
@@ -156,6 +172,17 @@ class GUI_EXPORT QgsGraduatedSymbolRendererWidget : public QgsRendererWidget, pr
     void keyPressEvent( QKeyEvent *event ) override;
 
   private:
+    enum MethodMode
+    {
+      ColorMode,
+      SizeMode
+    };
+
+    QgsExpressionContext createExpressionContext() const override;
+    void toggleMethodWidgets( MethodMode mode );
+
+    void clearParameterWidgets();
+
     std::unique_ptr< QgsGraduatedSymbolRenderer > mRenderer;
 
     std::unique_ptr< QgsSymbol > mGraduatedSymbol;
@@ -166,8 +193,10 @@ class GUI_EXPORT QgsGraduatedSymbolRendererWidget : public QgsRendererWidget, pr
 
     QgsRangeList mCopyBuffer;
 
-    QgsExpressionContext createExpressionContext() const override;
+    QDoubleValidator *mSymmetryPointValidator;
+
+    std::vector< std::unique_ptr< QgsAbstractProcessingParameterWidgetWrapper >> mParameterWidgetWrappers;
 };
 
 
-#endif // QGSGRADUATEDSYMBOLRENDERERV2WIDGET_H
+#endif // QGSGRADUATEDSYMBOLRENDERERWIDGET_H
