@@ -28,7 +28,7 @@
 #include <QVariant>
 #include <QIcon>
 
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgserror.h"
 #include "qgsobjectcustomproperties.h"
 #include "qgsrectangle.h"
@@ -38,6 +38,7 @@
 #include "qgslayermetadata.h"
 #include "qgsmaplayerstyle.h"
 #include "qgsreadwritecontext.h"
+#include "qgsdataprovider.h"
 
 class QgsAbstract3DRenderer;
 class QgsDataProvider;
@@ -45,6 +46,8 @@ class QgsMapLayerLegend;
 class QgsMapLayerRenderer;
 class QgsMapLayerStyleManager;
 class QgsProject;
+class QgsStyleEntityVisitorInterface;
+class QgsMapLayerTemporalProperties;
 
 class QDomDocument;
 class QKeyEvent;
@@ -55,6 +58,22 @@ class QPainter;
  */
 #define QGSCLIPBOARD_MAPLAYER_MIME "application/qgis.maplayer"
 
+
+/**
+ * \ingroup core
+ * Types of layers that can be added to a map
+ * \since QGIS 3.8
+ */
+enum class QgsMapLayerType SIP_MONKEYPATCH_SCOPEENUM_UNNEST( QgsMapLayer, LayerType ) : int
+  {
+  VectorLayer,
+  RasterLayer,
+  PluginLayer,
+  MeshLayer,      //!< Added in 3.2
+  VectorTileLayer, //!< Added in 3.14
+  AnnotationLayer, //!< Contains freeform, georeferenced annotations. Added in QGIS 3.16
+};
+
 /**
  * \ingroup core
  * Base class for all map layer types.
@@ -68,10 +87,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
     Q_PROPERTY( int autoRefreshInterval READ autoRefreshInterval WRITE setAutoRefreshInterval NOTIFY autoRefreshIntervalChanged )
     Q_PROPERTY( QgsLayerMetadata metadata READ metadata WRITE setMetadata NOTIFY metadataChanged )
     Q_PROPERTY( QgsCoordinateReferenceSystem crs READ crs WRITE setCrs NOTIFY crsChanged )
+    Q_PROPERTY( QgsMapLayerType type READ type CONSTANT )
+    Q_PROPERTY( bool isValid READ isValid NOTIFY isValidChanged )
 
 #ifdef SIP_RUN
     SIP_CONVERT_TO_SUBCLASS_CODE
-    QgsMapLayer *layer = qobject_cast<QgsMapLayer *>( sipCpp );
+    QgsMapLayer * layer = qobject_cast<QgsMapLayer *>( sipCpp );
 
     sipType = 0;
 
@@ -79,18 +100,24 @@ class CORE_EXPORT QgsMapLayer : public QObject
     {
       switch ( layer->type() )
       {
-        case QgsMapLayer::VectorLayer:
+        case QgsMapLayerType::VectorLayer:
           sipType = sipType_QgsVectorLayer;
           break;
-        case QgsMapLayer::RasterLayer:
+        case QgsMapLayerType::RasterLayer:
           sipType = sipType_QgsRasterLayer;
           break;
-        case QgsMapLayer::PluginLayer:
+        case QgsMapLayerType::PluginLayer:
           sipType = sipType_QgsPluginLayer;
           break;
-        case QgsMapLayer::MeshLayer:
+        case QgsMapLayerType::MeshLayer:
           sipType = sipType_QgsMeshLayer;
           break;
+        case QgsMapLayerType::VectorTileLayer:
+          sipType = sipType_QgsVectorTileLayer;
+          break;
+        case QgsMapLayerType::AnnotationLayer:
+          sipType = sipType_QgsAnnotationLayer;
+          break;
         default:
           sipType = nullptr;
           break;
@@ -101,15 +128,6 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
   public:
 
-    //! Types of layers that can be added to a map
-    enum LayerType
-    {
-      VectorLayer,
-      RasterLayer,
-      PluginLayer,
-      MeshLayer      //!< Added in 3.2
-    };
-
     /**
      * Maplayer has a style and a metadata property
      * \since QGIS 3.0
@@ -154,8 +172,11 @@ class CORE_EXPORT QgsMapLayer : public QObject
       Rendering          = 1 << 10, //!< Rendering: scale visibility, simplify method, opacity
       CustomProperties   = 1 << 11, //!< Custom properties (by plugins for instance)
       GeometryOptions    = 1 << 12, //!< Geometry validation configuration
+      Relations          = 1 << 13, //!< Relations
+      Temporal           = 1 << 14, //!< Temporal properties
+      Legend             = 1 << 15, //!< Legend settings (since QGIS 3.16)
       AllStyleCategories = LayerConfiguration | Symbology | Symbology3D | Labeling | Fields | Forms | Actions |
-                           MapTips | Diagrams | AttributeTable | Rendering | CustomProperties | GeometryOptions,
+                           MapTips | Diagrams | AttributeTable | Rendering | CustomProperties | GeometryOptions | Relations | Temporal | Legend,
     };
     Q_ENUM( StyleCategory )
     Q_DECLARE_FLAGS( StyleCategories, StyleCategory )
@@ -167,7 +188,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param name display name for the layer
      * \param source datasource of layer
      */
-    QgsMapLayer( QgsMapLayer::LayerType type = VectorLayer, const QString &name = QString(), const QString &source = QString() );
+    QgsMapLayer( QgsMapLayerType type = QgsMapLayerType::VectorLayer, const QString &name = QString(), const QString &source = QString() );
 
     ~QgsMapLayer() override;
 
@@ -187,7 +208,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Returns the type of the layer.
      */
-    QgsMapLayer::LayerType type() const;
+    QgsMapLayerType type() const;
 
     /**
      * Returns the flags for this layer.
@@ -200,7 +221,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns the flags for this layer.
-      \note Flags are options specified by the user used for the UI but are not preventing any API call.
+     * \note Flags are options specified by the user used for the UI but are not preventing any API call.
      * For instance, even if the Removable flag is not set, the layer can still be removed with the API
      * but the action will not be listed in the legend menu.
      * \since QGIS 3.4
@@ -231,12 +252,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QString name() const;
 
     /**
-     * Returns the layer's data provider.
+     * Returns the layer's data provider, it may be NULLPTR.
      */
     virtual QgsDataProvider *dataProvider();
 
     /**
-     * Returns the layer's data provider in a const-correct manner
+     * Returns the layer's data provider in a const-correct manner, it may be NULLPTR.
      * \note not available in Python bindings
      */
     virtual const QgsDataProvider *dataProvider() const SIP_SKIP;
@@ -254,7 +275,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  used by QGIS Server to identify the layer.
      * \see setShortName()
      */
-    QString shortName() const { return mShortName; }
+    QString shortName() const;
 
     /**
      * Sets the title of the layer
@@ -468,7 +489,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Returns the status of the layer. An invalid layer is one which has a bad datasource
      * or other problem. Child classes set this flag when initialized.
-     * \returns true if the layer is valid and can be accessed
+     * \returns TRUE if the layer is valid and can be accessed
      */
     bool isValid() const;
 
@@ -506,19 +527,42 @@ class CORE_EXPORT QgsMapLayer : public QObject
     */
     virtual void setSubLayerVisibility( const QString &name, bool visible );
 
-    //! Returns true if the layer can be edited.
+    //! Returns TRUE if the layer can be edited.
     virtual bool isEditable() const;
 
     /**
-     * Returns true if the layer is considered a spatial layer, ie it has some form of geometry associated with it.
+     * Returns TRUE if the layer is considered a spatial layer, ie it has some form of geometry associated with it.
      * \since QGIS 2.16
      */
     virtual bool isSpatial() const;
 
+    /**
+     * Returns TRUE if the layer is considered a temporary layer.
+     *
+     * These include memory-only layers such as those created by the "memory" data provider, layers
+     * stored inside a local temporary folder (such as the "/tmp" folder on Linux)
+     * or layer with temporary data (as temporary mesh layer dataset)
+     *
+     * \since QGIS 3.10.1
+     */
+    virtual bool isTemporary() const;
+
+    /**
+     * Flags which control project read behavior.
+     * \since QGIS 3.10
+     */
+    enum ReadFlag
+    {
+      FlagDontResolveLayers = 1 << 0, //!< Don't resolve layer paths or create data providers for layers.
+      FlagTrustLayerMetadata = 1 << 1, //!< Trust layer metadata. Improves layer load time by skipping expensive checks like primary key unicity, geometry type and srid and by using estimated metadata on layer load. Since QGIS 3.16
+    };
+    Q_DECLARE_FLAGS( ReadFlags, ReadFlag )
+
     /**
      * Sets state from DOM document
      * \param layerElement The DOM element corresponding to ``maplayer'' tag
      * \param context writing context (e.g. for conversion between relative and absolute paths)
+     * \param flags optional argument which can be used to control layer reading behavior.
      * \note
      *
      * The DOM node corresponds to a DOM document project file XML element read
@@ -529,9 +573,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *
      * Invoked by QgsProject::read().
      *
-     * \returns true if successful
+     * \returns TRUE if successful
      */
-    bool readLayerXml( const QDomElement &layerElement, QgsReadWriteContext &context );
+    bool readLayerXml( const QDomElement &layerElement, QgsReadWriteContext &context, QgsMapLayer::ReadFlags flags = QgsMapLayer::ReadFlags() );
 
     /**
      * Stores state in DOM node
@@ -548,7 +592,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *
      * Invoked by QgsProject::write().
      *
-     * \returns true if successful
+     * \returns TRUE if successful
      */
     bool writeLayerXml( QDomElement &layerElement, QDomDocument &document, const QgsReadWriteContext &context ) const;
 
@@ -563,20 +607,20 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \see customProperty()
      * \since QGIS 3.0
      */
-    QStringList customPropertyKeys() const;
+    Q_INVOKABLE QStringList customPropertyKeys() const;
 
     /**
      * Set a custom property for layer. Properties are stored in a map and saved in project file.
      * \see customProperty()
      * \see removeCustomProperty()
      */
-    void setCustomProperty( const QString &key, const QVariant &value );
+    Q_INVOKABLE void setCustomProperty( const QString &key, const QVariant &value );
 
     /**
      * Read a custom property from layer. Properties are stored in a map and saved in project file.
      * \see setCustomProperty()
     */
-    QVariant customProperty( const QString &value, const QVariant &defaultValue = QVariant() ) const;
+    Q_INVOKABLE QVariant customProperty( const QString &value, const QVariant &defaultValue = QVariant() ) const;
 
     /**
      * Set custom properties for layer. Current properties are dropped.
@@ -584,6 +628,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void setCustomProperties( const QgsObjectCustomProperties &properties );
 
+    /**
+     * Read all custom properties from layer. Properties are stored in a map and saved in project file.
+     * \see setCustomProperties
+     * \since QGIS 3.14
+     */
+    const QgsObjectCustomProperties &customProperties() const;
+
     /**
      * Remove a custom property from layer. Properties are stored in a map and saved in project file.
      * \see setCustomProperty()
@@ -599,13 +650,21 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns the layer's spatial reference system.
-    \since QGIS 1.4
+     * \since QGIS 1.4
      */
     QgsCoordinateReferenceSystem crs() const;
 
     //! Sets layer's spatial reference system
     void setCrs( const QgsCoordinateReferenceSystem &srs, bool emitSignal = true );
 
+    /**
+     * Returns the layer data provider coordinate transform context
+     * or a default transform context if the layer does not have a valid data provider.
+    * \since QGIS 3.8
+     */
+    QgsCoordinateTransformContext transformContext( ) const;
+
+
     /**
      * A convenience function to capitalize and format a layer \a name.
      *
@@ -634,7 +693,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Save the current metadata of this layer as the default metadata
      * (either as a .qmd file on disk or as a
      * record in the users style table in their personal qgis.db)
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to save the default metadata.
      * \returns a QString with any status messages
      * \since QGIS 3.0
@@ -650,7 +709,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * is a file and save to that, if that fails the qgis.db metadata
      * table will be used to create a metadata entry who's
      * key matches the URI.
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to save the default metadata.
      * \returns a QString with any status messages
      * \since QGIS 3.0
@@ -666,7 +725,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * is a file and load that, if that fails the qgis.db metadata
      * table will be consulted to see if there is a metadata who's
      * key matches the URI.
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to load the default metadata.
      * \returns a QString with any status messages
      * \since QGIS 3.0
@@ -677,19 +736,19 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Retrieve the default metadata for this layer if one
      * exists (either as a .qmd file on disk or as a
      * record in the users metadata table in their personal qgis.db)
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to load the default metadata.
      * \returns a QString with any status messages
      * \since QGIS 3.0
      */
-    QString loadDefaultMetadata( bool &resultFlag );
+    virtual QString loadDefaultMetadata( bool &resultFlag );
 
     /**
      * Retrieve a named metadata for this layer from a sqlite database.
      * \param db path to sqlite database
      * \param uri uri for table
      * \param qmd will be set to QMD xml metadata content from database
-     * \returns true if style was successfully loaded
+     * \returns TRUE if style was successfully loaded
      * \since QGIS 3.0
      */
     bool loadNamedMetadataFromDatabase( const QString &db, const QString &uri, QString &qmd );
@@ -698,7 +757,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Import the metadata of this layer from a QDomDocument
      * \param document source QDomDocument
      * \param errorMessage this QString will be initialized on error
-     * \returns true on success
+     * \returns TRUE on success
      * \since QGIS 3.0
      */
     bool importNamedMetadata( QDomDocument &document, QString &errorMessage );
@@ -716,7 +775,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Retrieve the default style for this layer if one
      * exists (either as a .qml file on disk or as a
      * record in the users style table in their personal qgis.db)
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to load the default style.
      * \returns a QString with any status messages
      * \see also loadNamedStyle();
@@ -732,7 +791,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * is a file and load that, if that fails the qgis.db styles
      * table will be consulted to see if there is a style who's
      * key matches the URI.
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to load the default style.
      * \param categories the style categories to be loaded.
      * \returns a QString with any status messages
@@ -745,7 +804,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param db path to sqlite database
      * \param uri uri for table
      * \param qml will be set to QML style content from database
-     * \returns true if style was successfully loaded
+     * \returns TRUE if style was successfully loaded
      */
     virtual bool loadNamedStyleFromDatabase( const QString &db, const QString &uri, QString &qml SIP_OUT );
 
@@ -755,7 +814,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param errorMsg this QString will be initialized on error
      * during the execution of readSymbology
      * \param categories the style categories to import
-     * \returns true on success
+     * \returns TRUE on success
      * \since QGIS 2.8
      */
     virtual bool importNamedStyle( QDomDocument &doc, QString &errorMsg SIP_OUT,
@@ -785,7 +844,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Save the properties of this layer as the default style
      * (either as a .qml file on disk or as a
      * record in the users style table in their personal qgis.db)
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to save the default style.
      * \returns a QString with any status messages
      * \see loadNamedStyle() and \see saveNamedStyle()
@@ -801,7 +860,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * is a file and save to that, if that fails the qgis.db styles
      * table will be used to create a style entry who's
      * key matches the URI.
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to save the default style.
      * \param categories the style categories to be saved.
      * \returns a QString with any status messages
@@ -812,7 +871,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Saves the properties of this layer to an SLD format file.
      * \param uri uri of destination for exported SLD file.
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * the SLD file could not be generated
      * \returns a string with any status or error messages
      * \see loadSldStyle()
@@ -822,7 +881,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Attempts to style the layer using the formatting from an SLD type file.
      * \param uri uri of source SLD file
-     * \param resultFlag a reference to a flag that will be set to false if
+     * \param resultFlag a reference to a flag that will be set to FALSE if
      * the SLD file could not be loaded
      * \returns a string with any status or error messages
      * \see saveSldStyle()
@@ -830,7 +889,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     virtual QString loadSldStyle( const QString &uri, bool &resultFlag );
 
     virtual bool readSld( const QDomNode &node, QString &errorMessage )
-    { Q_UNUSED( node ); errorMessage = QStringLiteral( "Layer type %1 not supported" ).arg( type() ); return false; }
+    { Q_UNUSED( node ) errorMessage = QStringLiteral( "Layer type %1 not supported" ).arg( static_cast<int>( type() ) ); return false; }
 
 
 
@@ -840,7 +899,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param errorMessage reference to string that will be updated with any error messages
      * \param context reading context (used for transform from relative to absolute paths)
      * \param categories the style categories to be read
-     * \returns true in case of success.
+     * \returns TRUE in case of success.
      */
     virtual bool readSymbology( const QDomNode &node, QString &errorMessage,
                                 QgsReadWriteContext &context, StyleCategories categories = AllStyleCategories ) = 0;
@@ -851,8 +910,8 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param errorMessage reference to string that will be updated with any error messages
      * \param context reading context (used for transform from relative to absolute paths)
      * \param categories the style categories to be read
-     * \returns true in case of success.
-     * \note To be implemented in subclasses. Default implementation does nothing and returns false.
+     * \returns TRUE in case of success.
+     * \note To be implemented in subclasses. Default implementation does nothing and returns FALSE.
      * \since QGIS 2.16
      */
     virtual bool readStyle( const QDomNode &node, QString &errorMessage,
@@ -866,7 +925,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  \param context writing context (used for transform from absolute to relative paths)
      *  \param categories the style categories to be written
      *  \note There is a confusion of terms with the GUI. This method actually writes what is called a style in the application.
-     *  \returns true in case of success.
+     *  \returns TRUE in case of success.
      */
     virtual bool writeSymbology( QDomNode &node, QDomDocument &doc, QString &errorMessage, const QgsReadWriteContext &context,
                                  StyleCategories categories = AllStyleCategories ) const = 0;
@@ -878,14 +937,37 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  \param errorMessage reference to string that will be updated with any error messages
      *  \param context writing context (used for transform from absolute to relative paths)
      *  \param categories the style categories to be written
-     *  \returns true in case of success.
-     *  \note To be implemented in subclasses. Default implementation does nothing and returns false.
+     *  \returns TRUE in case of success.
+     *  \note To be implemented in subclasses. Default implementation does nothing and returns FALSE.
      *  \note There is a confusion of terms with the GUI. This method actually writes what is known as the symbology in the application.
      *  \since QGIS 2.16
      */
     virtual bool writeStyle( QDomNode &node, QDomDocument &doc, QString &errorMessage, const QgsReadWriteContext &context,
                              StyleCategories categories = AllStyleCategories ) const;
 
+
+    /**
+     * Updates the data source of the layer. The layer's renderer and legend will be preserved only
+     * if the geometry type of the new data source matches the current geometry type of the layer.
+     *
+     * Subclasses should override this method: default implementation does nothing.
+     *
+     * \param dataSource new layer data source
+     * \param baseName base name of the layer
+     * \param provider provider string
+     * \param options provider options
+     * \param loadDefaultStyleFlag set to TRUE to reset the layer's style to the default for the
+     * data source
+     * \see dataSourceChanged()
+     * \since QGIS 3.6
+     */
+    virtual void setDataSource( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, bool loadDefaultStyleFlag = false );
+
+    /**
+     * Returns the provider type (provider key) for this layer
+     */
+    QString providerType() const;
+
     //! Returns pointer to layer's undo stack
     QUndoStack *undoStack();
 
@@ -917,13 +999,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Assign a legend controller to the map layer. The object will be responsible for providing legend items.
-     * \param legend Takes ownership of the object. Can be null pointer
+     * \param legend Takes ownership of the object. Can be NULLPTR.
      * \since QGIS 2.6
      */
     void setLegend( QgsMapLayerLegend *legend SIP_TRANSFER );
 
     /**
-     * Can be null.
+     * Can be NULLPTR.
      * \since QGIS 2.6
      */
     QgsMapLayerLegend *legend() const;
@@ -941,7 +1023,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     void setRenderer3D( QgsAbstract3DRenderer *renderer SIP_TRANSFER );
 
     /**
-     * Returns 3D renderer associated with the layer. May be null.
+     * Returns 3D renderer associated with the layer. May be NULLPTR.
      * \since QGIS 3.0
      */
     QgsAbstract3DRenderer *renderer3D() const;
@@ -949,7 +1031,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Tests whether the layer should be visible at the specified \a scale.
      *  The \a scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
-     * \returns true if the layer is visible at the given scale.
+     * \returns TRUE if the layer is visible at the given scale.
      * \see minimumScale()
      * \see maximumScale()
      * \see hasScaleBasedVisibility()
@@ -961,7 +1043,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Returns the minimum map scale (i.e. most "zoomed out" scale) at which the layer will be visible.
      * The scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A scale of 0 indicates no minimum scale visibility.
-     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to true.
+     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to TRUE.
      * \see setMinimumScale()
      * \see maximumScale()
      * \see hasScaleBasedVisibility()
@@ -973,7 +1055,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Returns the maximum map scale (i.e. most "zoomed in" scale) at which the layer will be visible.
      * The scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A scale of 0 indicates no maximum scale visibility.
-     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to true.
+     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to TRUE.
      * \see setMaximumScale()
      * \see minimumScale()
      * \see hasScaleBasedVisibility()
@@ -983,7 +1065,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns whether scale based visibility is enabled for the layer.
-     * \returns true if scale based visibility is enabled
+     * \returns TRUE if scale based visibility is enabled
      * \see minimumScale()
      * \see maximumScale()
      * \see setScaleBasedVisibility()
@@ -992,7 +1074,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     bool hasScaleBasedVisibility() const;
 
     /**
-     * Returns true if auto refresh is enabled for the layer.
+     * Returns TRUE if auto refresh is enabled for the layer.
      * \see autoRefreshInterval()
      * \see setAutoRefreshEnabled()
      * \since QGIS 3.0
@@ -1001,7 +1083,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns the auto refresh interval (in milliseconds). Note that
-     * auto refresh is only active when hasAutoRefreshEnabled() is true.
+     * auto refresh is only active when hasAutoRefreshEnabled() is TRUE.
      * \see hasAutoRefreshEnabled()
      * \see setAutoRefreshInterval()
      * \since QGIS 3.0
@@ -1071,19 +1153,62 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QString refreshOnNotifyMessage() const { return mRefreshOnNofifyMessage; }
 
     /**
-     * Returns true if the refresh on provider nofification is enabled
+     * Returns TRUE if the refresh on provider nofification is enabled
      *
      * \since QGIS 3.0
      */
     bool isRefreshOnNotifyEnabled() const { return mIsRefreshOnNofifyEnabled; }
 
+    /**
+     * Returns the XML properties of the original layer as they were when the layer
+     * was first read from the project file. In case of new layers this is normally empty.
+     *
+     * The storage format for the XML is qlr
+     *
+     * \since QGIS 3.6
+     */
+    QString originalXmlProperties() const;
+
+    /**
+     * Sets the original XML properties for the layer to  \a originalXmlProperties
+     *
+     * The storage format for the XML is qlr
+     *
+     * \since QGIS 3.6
+     */
+    void setOriginalXmlProperties( const QString &originalXmlProperties );
+
+    /**
+     * Generates an unique identifier for this layer, the generate ID is prefixed by \a layerName
+     * \since QGIS 3.8
+     */
+    static QString generateId( const QString &layerName );
+
+    /**
+     * Accepts the specified symbology \a visitor, causing it to visit all symbols associated
+     * with the layer.
+     *
+     * Returns TRUE if the visitor should continue visiting other objects, or FALSE if visiting
+     * should be canceled.
+     *
+     * \since QGIS 3.10
+     */
+    virtual bool accept( QgsStyleEntityVisitorInterface *visitor ) const;
+
+    /**
+     * Returns the layer's temporal properties. This may be NULLPTR, depending on the layer type.
+     *
+     * \since QGIS 3.14
+     */
+    virtual QgsMapLayerTemporalProperties *temporalProperties() { return nullptr; }
+
   public slots:
 
     /**
      * Sets the minimum map \a scale (i.e. most "zoomed out" scale) at which the layer will be visible.
      * The \a scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A \a scale of 0 indicates no minimum scale visibility.
-     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to true.
+     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to TRUE.
      * \see minimumScale()
      * \see setMaximumScale()
      * \see setScaleBasedVisibility()
@@ -1094,7 +1219,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Sets the maximum map \a scale (i.e. most "zoomed in" scale) at which the layer will be visible.
      * The \a scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A \a scale of 0 indicates no maximum scale visibility.
-     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to true.
+     * \note Scale based visibility is only used if setScaleBasedVisibility() is set to TRUE.
      * \see maximumScale()
      * \see setMinimumScale()
      * \see setScaleBasedVisibility()
@@ -1103,7 +1228,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Sets whether scale based visibility is enabled for the layer.
-     * \param enabled set to true to enable scale based visibility
+     * \param enabled set to TRUE to enable scale based visibility
      * \see setMinimumScale
      * \see setMaximumScale
      * \see hasScaleBasedVisibility
@@ -1113,7 +1238,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Will advise the map canvas (and any other interested party) that this layer requires to be repainted.
      * Will emit a repaintRequested() signal.
-     * If \a deferredUpdate is true then the layer will only be repainted when the canvas is next
+     * If \a deferredUpdate is TRUE then the layer will only be repainted when the canvas is next
      * re-rendered, and will not trigger any canvas redraws itself.
      *
      * \note in 2.6 function moved from vector/raster subclasses to QgsMapLayer
@@ -1131,7 +1256,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \see dependencies()
      *
      * \param layers set of QgsMapLayerDependency. Only user-defined dependencies will be added
-     * \returns false if a dependency cycle has been detected
+     * \returns FALSE if a dependency cycle has been detected
      * \since QGIS 3.0
      */
     virtual bool setDependencies( const QSet<QgsMapLayerDependency> &layers );
@@ -1144,7 +1269,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     void setRefreshOnNotifyEnabled( bool enabled );
 
     /**
-     * Set the notification message that triggers repaine
+     * Set the notification message that triggers repaint
      * If refresh on notification is enabled, the notification will triggerRepaint only
      * if the notification message is equal to \param message
      *
@@ -1152,8 +1277,31 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void setRefreshOnNofifyMessage( const QString &message ) { mRefreshOnNofifyMessage = message; }
 
+    /**
+     * Sets the coordinate transform context to \a transformContext
+     *
+     * \since QGIS 3.8
+     */
+    virtual void setTransformContext( const QgsCoordinateTransformContext &transformContext ) = 0;
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsMapLayer: '%1' (%2)>" ).arg( sipCpp->name(), sipCpp->dataProvider() ? sipCpp->dataProvider()->name() : QStringLiteral( "Invalid" ) );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
   signals:
 
+    /**
+     * Emitted when all layers are loaded and references can be resolved,
+     * just before the references of this layer are resolved.
+     *
+     * \since QGIS 3.10
+     */
+    void beforeResolveReferences( QgsProject *project );
+
     //! Emit a signal with status (e.g. to be caught by QgisApp and display a msg on status bar)
     void statusChanged( const QString &status );
 
@@ -1170,7 +1318,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * By emitting this signal the layer tells that either appearance or content have been changed
      * and any view showing the rendered layer should refresh itself.
-     * If \a deferredUpdate is true then the layer will only be repainted when the canvas is next
+     * If \a deferredUpdate is TRUE then the layer will only be repainted when the canvas is next
      * re-rendered, and will not trigger any canvas redraws itself.
      */
     void repaintRequested( bool deferredUpdate = false );
@@ -1253,6 +1401,30 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void flagsChanged();
 
+    /**
+     * Emitted whenever the layer's data source has been changed.
+     *
+     * \see setDataSource()
+     *
+     * \since QGIS 3.5
+     */
+    void dataSourceChanged();
+
+    /**
+     * Emitted when a style has been loaded
+     * \param categories style categories
+     * \since QGIS 3.12
+     */
+    void styleLoaded( QgsMapLayer::StyleCategories categories );
+
+    /**
+     * Emitted when the validity of this layer changed.
+     *
+     * \since QGIS 3.16
+     */
+    void isValidChanged();
+
+
   private slots:
 
     void onNotifiedTriggerRepaint( const QString &message );
@@ -1269,7 +1441,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! Sets the extent
     virtual void setExtent( const QgsRectangle &rect );
 
-    //! Sets whether layer is valid or not - should be used in constructor.
+    //! Sets whether layer is valid or not
     void setValid( bool valid );
 
     /**
@@ -1313,8 +1485,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Read custom properties from project file.
-      \param layerNode note to read from
-      \param keyStartsWith reads only properties starting with the specified string (or all if the string is empty)*/
+     * \param layerNode note to read from
+     * \param keyStartsWith reads only properties starting with the specified string (or all if the string is empty)
+    */
     void readCustomProperties( const QDomNode &layerNode, const QString &keyStartsWith = QString() );
 
     //! Write custom properties to project file.
@@ -1340,6 +1513,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
     void readCommonStyle( const QDomElement &layerElement, const QgsReadWriteContext &context,
                           StyleCategories categories = AllStyleCategories );
 
+    //! Sets the \a providerType (provider key)
+    void setProviderType( const QString &providerType );
+
 #ifndef SIP_RUN
 #if 0
     //! Debugging member - invoked when a connect() is made to this object
@@ -1394,12 +1570,31 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! List of layers that may modify this layer on modification
     QSet<QgsMapLayerDependency> mDependencies;
 
-    //! Checks whether a new set of dependencies will introduce a cycle
-    bool hasDependencyCycle( const QSet<QgsMapLayerDependency> &layers ) const;
+    /**
+     * Checks whether a new set of dependencies will introduce a cycle
+     * this method is now deprecated and always return FALSE, because circular dependencies are now correctly managed.
+     * \deprecated since QGIS 3.10
+     */
+    Q_DECL_DEPRECATED bool hasDependencyCycle( const QSet<QgsMapLayerDependency> & ) const {return false;}
 
     bool mIsRefreshOnNofifyEnabled = false;
     QString mRefreshOnNofifyMessage;
 
+    //! Data provider key (name of the data provider)
+    QString mProviderKey;
+
+    //TODO QGIS 4 - move to readXml as a new argument (breaks API)
+
+    //! Read flags. It's up to the subclass to respect these when restoring state from XML
+    QgsMapLayer::ReadFlags mReadFlags = QgsMapLayer::ReadFlags();
+
+    /**
+     * TRUE if the layer's CRS should be validated and invalid CRSes are not permitted.
+     *
+     * \since QGIS 3.10
+     */
+    bool mShouldValidateCrs = true;
+
   private:
 
     virtual QString baseURI( PropertyType type ) const;
@@ -1410,21 +1605,22 @@ class CORE_EXPORT QgsMapLayer : public QObject
     bool loadNamedPropertyFromDatabase( const QString &db, const QString &uri, QString &xml, QgsMapLayer::PropertyType type );
 
     /**
-     * This method returns true by default but can be overwritten to specify
+     * This method returns TRUE by default but can be overwritten to specify
      * that a certain layer is writable.
      */
     virtual bool isReadOnly() const;
 
     /**
      * Layer's spatial reference system.
-        private to make sure setCrs must be used and crsChanged() is emitted */
+     * private to make sure setCrs must be used and crsChanged() is emitted.
+    */
     QgsCoordinateReferenceSystem mCRS;
 
     //! Unique ID of this layer - used to refer to this layer in map layer registry
     QString mID;
 
     //! Type of the layer (e.g., vector, raster)
-    QgsMapLayer::LayerType mLayerType;
+    QgsMapLayerType mLayerType;
 
     LayerFlags mFlags = LayerFlags( Identifiable | Removable | Searchable );
 
@@ -1443,7 +1639,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! A flag that tells us whether to use the above vars to restrict layer visibility
     bool mScaleBasedVisibility = false;
 
-    //! Collection of undoable operations for this layer. *
+    //! Collection of undoable operations for this layer.
     QUndoStack *mUndoStack = nullptr;
 
     QUndoStack *mUndoStackStyles = nullptr;
@@ -1454,7 +1650,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! Controller of legend items of this layer
     QgsMapLayerLegend *mLegend = nullptr;
 
-    //! Manager of multiple styles available for a layer (may be null)
+    //! Manager of multiple styles available for a layer (may be NULLPTR)
     QgsMapLayerStyleManager *mStyleManager = nullptr;
 
     //! Timer for triggering automatic refreshes of the layer
@@ -1465,11 +1661,21 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! Renderer for 3D views
     QgsAbstract3DRenderer *m3DRenderer = nullptr;
 
+    /**
+     * Stores the original XML properties of the layer when loaded from the project
+     *
+     * This information can be used to pass through the bad layers or to reset changes on a good layer
+     */
+    QString mOriginalXmlProperties;
+
+    //! To avoid firing multiple time repaintRequested signal on circular layer circular dependencies
+    bool mRepaintRequestedFired = false;
 };
 
 Q_DECLARE_METATYPE( QgsMapLayer * )
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapLayer::LayerFlags )
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapLayer::StyleCategories )
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapLayer::ReadFlags )
 
 
 #ifndef SIP_RUN
