@@ -21,15 +21,12 @@ __author__ = 'Victor Olaya'
 __date__ = 'February 2015'
 __copyright__ = '(C) 2014-2015, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 import stat
 import shutil
 import shlex
 import subprocess
 import os
+import sys
 
 from qgis.core import (Qgis,
                        QgsApplication,
@@ -43,16 +40,15 @@ from processing.algs.gdal.GdalUtils import GdalUtils
 
 
 class Grass7Utils:
-
     GRASS_REGION_XMIN = 'GRASS7_REGION_XMIN'
     GRASS_REGION_YMIN = 'GRASS7_REGION_YMIN'
     GRASS_REGION_XMAX = 'GRASS7_REGION_XMAX'
     GRASS_REGION_YMAX = 'GRASS7_REGION_YMAX'
     GRASS_REGION_CELLSIZE = 'GRASS7_REGION_CELLSIZE'
-    GRASS_FOLDER = 'GRASS7_FOLDER'
     GRASS_LOG_COMMANDS = 'GRASS7_LOG_COMMANDS'
     GRASS_LOG_CONSOLE = 'GRASS7_LOG_CONSOLE'
     GRASS_HELP_PATH = 'GRASS_HELP_PATH'
+    GRASS_USE_REXTERNAL = 'GRASS_USE_REXTERNAL'
     GRASS_USE_VEXTERNAL = 'GRASS_USE_VEXTERNAL'
 
     # TODO Review all default options formats
@@ -132,8 +128,6 @@ class Grass7Utils:
         Find GRASS binary path on the operating system.
         Sets global variable Grass7Utils.command
         """
-        cmdList = ["grass74", "grass72", "grass71", "grass70", "grass",
-                   "grass74.sh", "grass72.sh", "grass71.sh", "grass70.sh", "grass.sh"]
 
         def searchFolder(folder):
             """
@@ -153,6 +147,24 @@ class Grass7Utils:
         path = Grass7Utils.grassPath()
         command = None
 
+        vn = os.path.join(path, "etc", "VERSIONNUMBER")
+        if os.path.isfile(vn):
+            with open(vn, "r") as f:
+                major, minor, patch = f.readlines()[0].split(' ')[0].split('.')
+                if patch != 'svn':
+                    patch = ''
+                cmdList = [
+                    "grass{}{}{}".format(major, minor, patch),
+                    "grass",
+                    "grass{}{}{}.sh".format(major, minor, patch),
+                    "grass.sh"
+                ]
+        else:
+            cmdList = [
+                "grass78", "grass76", "grass74", "grass72", "grass70", "grass",
+                "grass78.sh", "grass76.sh", "grass74.sh", "grass72.sh", "grass70.sh", "grass.sh"
+            ]
+
         # For MS-Windows there is a difference between GRASS Path and GRASS binary
         if isWindows():
             # If nothing found, use OSGEO4W or QgsPrefix:
@@ -166,7 +178,7 @@ class Grass7Utils:
             # Search in grassPath
             command = searchFolder(path)
 
-        # Under GNU/Linux or if everything has failed, use shutil
+        # If everything has failed, use shutil
         if not command:
             for cmd in cmdList:
                 testBin = shutil.which(cmd)
@@ -176,7 +188,7 @@ class Grass7Utils:
 
         if command:
             Grass7Utils.command = command
-            if path is '':
+            if path == '':
                 Grass7Utils.path = os.path.dirname(command)
 
         return command
@@ -193,29 +205,24 @@ class Grass7Utils:
         if not isWindows() and not isMac():
             return ''
 
-        if isMac():
-            folder = ProcessingConfig.getSetting(Grass7Utils.GRASS_FOLDER) or ''
-            if not os.path.exists(folder):
-                folder = None
-        else:
-            folder = None
-
-        if folder is None:
-            # Under MS-Windows, we use OSGEO4W or QGIS Path for folder
-            if isWindows():
-                if "OSGEO4W_ROOT" in os.environ:
-                    testfolder = os.path.join(str(os.environ['OSGEO4W_ROOT']), "apps")
-                else:
-                    testfolder = str(QgsApplication.prefixPath())
-                testfolder = os.path.join(testfolder, 'grass')
+        folder = None
+        # Under MS-Windows, we use GISBASE or QGIS Path for folder
+        if isWindows():
+            if "GISBASE" in os.environ:
+                folder = os.environ["GISBASE"]
+            else:
+                testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()), 'grass')
                 if os.path.isdir(testfolder):
-                    grassfolders = sorted([f for f in os.listdir(testfolder) if f.startswith("grass-7.") and os.path.isdir(os.path.join(testfolder, f))], reverse=True, key=lambda x: [int(v) for v in x[len("grass-"):].split('.')])
+                    grassfolders = sorted([f for f in os.listdir(testfolder) if f.startswith("grass-7.") and os.path.isdir(os.path.join(testfolder, f))], reverse=True, key=lambda x: [int(v) for v in x[len("grass-"):].split('.') if v != 'svn'])
                     if grassfolders:
                         folder = os.path.join(testfolder, grassfolders[0])
-            elif isMac():
-                # For MacOSX, we scan some well-known directories
-                # Start with QGIS bundle
-                for version in ['', '7', '70', '71', '72', '74']:
+        elif isMac():
+            # For MacOSX, first check environment
+            if "GISBASE" in os.environ:
+                folder = os.environ["GISBASE"]
+            else:
+                # Find grass folder if it exists inside QGIS bundle
+                for version in ['', '7', '78', '76', '74', '72', '71', '70']:
                     testfolder = os.path.join(str(QgsApplication.prefixPath()),
                                               'grass{}'.format(version))
                     if os.path.isdir(testfolder):
@@ -223,7 +230,7 @@ class Grass7Utils:
                         break
                     # If nothing found, try standalone GRASS installation
                     if folder is None:
-                        for version in ['0', '1', '2', '4']:
+                        for version in ['8', '6', '4', '2', '1', '0']:
                             testfolder = '/Applications/GRASS-7.{}.app/Contents/MacOS'.format(version)
                             if os.path.isdir(testfolder):
                                 folder = testfolder
@@ -356,10 +363,14 @@ class Grass7Utils:
         # QgsMessageLog.logMessage('exec: {}'.format(command), 'DEBUG', Qgis.Info)
 
         # For MS-Windows, we need to hide the console window.
+        kw = {}
         if isWindows():
             si = subprocess.STARTUPINFO()
             si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
             si.wShowWindow = subprocess.SW_HIDE
+            kw['startupinfo'] = si
+            if sys.version_info >= (3, 6):
+                kw['encoding'] = "cp{}".format(Grass7Utils.getWindowsCodePage())
 
         with subprocess.Popen(
                 command,
@@ -369,8 +380,7 @@ class Grass7Utils:
                 stderr=subprocess.STDOUT,
                 universal_newlines=True,
                 env=grassenv,
-                encoding="cp{}".format(Grass7Utils.getWindowsCodePage()) if isWindows() else None,
-                startupinfo=si if isWindows() else None
+                **kw
         ) as proc:
             for line in iter(proc.stdout.readline, ''):
                 if 'GRASS_INFO_PERCENT' in line:
@@ -387,6 +397,9 @@ class Grass7Utils:
                     elif 'Segmentation fault' in line:
                         feedback.reportError(line.strip())
                         feedback.reportError('\n' + Grass7Utils.tr('GRASS command crashed :( Try a different set of input parameters and consult the GRASS algorithm manual for more information.') + '\n')
+                        if ProcessingConfig.getSetting(Grass7Utils.GRASS_USE_REXTERNAL):
+                            feedback.reportError(Grass7Utils.tr(
+                                'Suggest disabling the experimental "use r.external" option from the Processing GRASS Provider options.') + '\n')
                         if ProcessingConfig.getSetting(Grass7Utils.GRASS_USE_VEXTERNAL):
                             feedback.reportError(Grass7Utils.tr(
                                 'Suggest disabling the experimental "use v.external" option from the Processing GRASS Provider options.') + '\n')
@@ -401,10 +414,14 @@ class Grass7Utils:
         if not grassOutDone and outputCommands:
             command, grassenv = Grass7Utils.prepareGrassExecution(outputCommands)
             # For MS-Windows, we need to hide the console window.
+            kw = {}
             if isWindows():
                 si = subprocess.STARTUPINFO()
                 si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                 si.wShowWindow = subprocess.SW_HIDE
+                kw['startupinfo'] = si
+                if sys.version_info >= (3, 6):
+                    kw['encoding'] = "cp{}".format(Grass7Utils.getWindowsCodePage())
             with subprocess.Popen(
                     command,
                     shell=False,
@@ -413,8 +430,7 @@ class Grass7Utils:
                     stderr=subprocess.STDOUT,
                     universal_newlines=True,
                     env=grassenv,
-                    encoding="cp{}".format(Grass7Utils.getWindowsCodePage()) if isWindows() else None,
-                    startupinfo=si if isWindows() else None
+                    **kw
             ) as proc:
                 for line in iter(proc.stdout.readline, ''):
                     if 'GRASS_INFO_PERCENT' in line:
@@ -449,7 +465,7 @@ class Grass7Utils:
     # the layers.
     @staticmethod
     def endGrassSession():
-        #shutil.rmtree(Grass7Utils.grassMapsetFolder(), True)
+        # shutil.rmtree(Grass7Utils.grassMapsetFolder(), True)
         Grass7Utils.sessionRunning = False
         Grass7Utils.sessionLayers = {}
         Grass7Utils.projectionSet = False
@@ -544,7 +560,7 @@ class Grass7Utils:
             return 'https://grass.osgeo.org/grass{}/manuals/'.format(version)
         else:
             # GRASS not available!
-            return 'https://grass.osgeo.org/grass72/manuals/'
+            return 'https://grass.osgeo.org/grass78/manuals/'
 
     @staticmethod
     def getSupportedOutputRasterExtensions():
