@@ -21,10 +21,6 @@ __author__ = 'Michael Minn'
 __date__ = 'May 2010'
 __copyright__ = '(C) 2010, Michael Minn'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QgsExpression,
                        QgsVectorLayer,
@@ -48,7 +44,7 @@ class SelectByAttribute(QgisAlgorithm):
     OUTPUT = 'OUTPUT'
 
     OPERATORS = ['=',
-                 '≠',
+                 '<>',
                  '>',
                  '>=',
                  '<',
@@ -76,15 +72,15 @@ class SelectByAttribute(QgisAlgorithm):
         super().__init__()
 
     def flags(self):
-        return super().flags() | QgsProcessingAlgorithm.FlagNoThreading
+        return super().flags() | QgsProcessingAlgorithm.FlagNoThreading | QgsProcessingAlgorithm.FlagNotAvailableInStandaloneTool
 
     def initAlgorithm(self, config=None):
         self.operators = ['=',
                           '≠',
                           '>',
-                          '>=',
+                          '≥',
                           '<',
-                          '<=',
+                          '≤',
                           self.tr('begins with'),
                           self.tr('contains'),
                           self.tr('is null'),
@@ -104,14 +100,14 @@ class SelectByAttribute(QgisAlgorithm):
                                                       self.tr('Selection attribute'),
                                                       parentLayerParameterName=self.INPUT))
         self.addParameter(QgsProcessingParameterEnum(self.OPERATOR,
-                                                     self.tr('Operator'), self.operators))
+                                                     self.tr('Operator'), self.operators, defaultValue=0))
         self.addParameter(QgsProcessingParameterString(self.VALUE,
                                                        self.tr('Value'),
                                                        optional=True))
         self.addParameter(QgsProcessingParameterEnum(self.METHOD,
                                                      self.tr('Modify current selection by'),
                                                      self.methods,
-                                                     0))
+                                                     defaultValue=0))
 
         self.addOutput(QgsProcessingOutputVectorLayer(self.OUTPUT, self.tr('Selected (attribute)')))
 
