@@ -21,10 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 import sys
 
 from qgis.core import (QgsExpressionContext,
@@ -42,6 +38,7 @@ from qgis.core import (QgsExpressionContext,
                        QgsProcessingOutputHtml,
                        QgsProcessingOutputNumber,
                        QgsProcessingOutputString,
+                       QgsProcessingOutputBoolean,
                        QgsProcessingOutputFolder,
                        QgsProcessingOutputMultipleLayers)
 
@@ -72,29 +69,31 @@ def getOutputFromString(s):
                 out = QgsProcessingOutputMapLayer(name, description)
             elif token.lower().strip() == 'outputmultilayers':
                 out = QgsProcessingOutputMultipleLayers(name, description)
-#            elif token.lower().strip() == 'vector point':
-#                out = OutputVector(datatype=[dataobjects.TYPE_VECTOR_POINT])
-#            elif token.lower().strip() == 'vector line':
-#                out = OutputVector(datatype=[OutputVector.TYPE_VECTOR_LINE])
-#            elif token.lower().strip() == 'vector polygon':
-#                out = OutputVector(datatype=[OutputVector.TYPE_VECTOR_POLYGON])
-#            elif token.lower().strip().startswith('table'):
-#                out = OutputTable()
+            #            elif token.lower().strip() == 'vector point':
+            #                out = OutputVector(datatype=[dataobjects.TYPE_VECTOR_POINT])
+            #            elif token.lower().strip() == 'vector line':
+            #                out = OutputVector(datatype=[OutputVector.TYPE_VECTOR_LINE])
+            #            elif token.lower().strip() == 'vector polygon':
+            #                out = OutputVector(datatype=[OutputVector.TYPE_VECTOR_POLYGON])
+            #            elif token.lower().strip().startswith('table'):
+            #                out = OutputTable()
             elif token.lower().strip().startswith('outputhtml'):
                 out = QgsProcessingOutputHtml(name, description)
-#            elif token.lower().strip().startswith('file'):
-#                out = OutputFile()
-#                ext = token.strip()[len('file') + 1:]
-#                if ext:
-#                    out.ext = ext
+            #            elif token.lower().strip().startswith('file'):
+            #                out = OutputFile()
+            #                ext = token.strip()[len('file') + 1:]
+            #                if ext:
+            #                    out.ext = ext
             elif token.lower().strip().startswith('outputfolder'):
                 out = QgsProcessingOutputFolder(name, description)
             elif token.lower().strip().startswith('outputnumber'):
                 out = QgsProcessingOutputNumber(name, description)
             elif token.lower().strip().startswith('outputstring'):
                 out = QgsProcessingOutputString(name, description)
-#            elif token.lower().strip().startswith('extent'):
-#                out = OutputExtent()
+            elif token.lower().strip().startswith('outputboolean'):
+                out = QgsProcessingOutputBoolean(name, description)
+            #            elif token.lower().strip().startswith('extent'):
+            #                out = OutputExtent()
 
             return out
     except:
